library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity video_processing  is
  port (
    IPD              : in  std_logic_vector(7 downto 0);
    HPD              : in  std_logic_vector(7 downto 0);
    IDQ              : in  std_logic;
    ICLK             : in  std_logic;
    ICLK_OUT         : out std_logic;
    pix_count        : in  std_logic_vector(10 downto 0);
    line_count       : in  std_logic_vector(15 downto 0);
    block_count      : out std_logic_vector(15 downto 0);
    load             : out  std_logic_vector(4 downto 0);
    reset            : in  std_logic;
    threshold        : in  std_logic_vector(31 downto 0)
    );
end video_processing ;

architecture imp of video_processing is

signal alt_counter : std_logic_vector(1 downto 0) := "00";
signal alt_count : std_logic := '1';    -- use counter half ICLK speed
signal n_count : std_logic := '1';      -- used to load pix_hold register

signal data_in : std_logic_vector( 15 downto 0 );  -- input to pix_hold reg
signal pixel_one : std_logic_vector(15 downto 0);  -- output of pix_hold reg

signal thresh_data : std_logic_vector( 31 downto 0);  -- input to thresh reg
signal thresh_comp : std_logic_vector( 31 downto 0);  -- output of thresh reg

signal chrom11, lum2, chrom1, chrom2 : std_logic := '0';  -- thresholding signals
signal on_pix : std_logic := '0';

signal grid1, grid2, grid3, grid4 : std_logic_vector(15 downto 0) := X"0000";-- "on" pixel counts
signal line_count_seg : std_logic_vector(15 downto 0);  -- line count per quarter
signal addr_marker  : std_logic_vector(1 downto 0);

component regis16 is
  port( rst, clk, load: in std_logic;
	input: in std_logic_vector( 15 downto 0 );
	output: out std_logic_vector( 15 downto 0 ));
end component;

component regis32 is
  port( rst, clk, load: in std_logic;
	input: in std_logic_vector( 31 downto 0 );
	output: out std_logic_vector( 31 downto 0 ));
end component;

begin

  pix_hold : regis16 port map (
    rst    => reset,
    clk    => ICLK,
    load   => n_count,
    input  => data_in,
    output => pixel_one);

  thresh_reg : regis32 port map (
    rst    => reset,
    clk    => ICLK,
    load   => alt_count,
    input  => thresh_data,
    output => thresh_comp);

-- mark every other pixel, so that we can process two pixels at a time
every_other: process(ICLK)
  begin
    if ICLK'event and ICLK = '1' and IDQ = '1' then
        alt_counter <= alt_counter + 1;
    end if;
  end process;

-- combine adjacent pixel data
two_pix : process(alt_count)
  begin
    if alt_count'event and alt_count = '1' then
      thresh_data <= pixel_one & data_in;
    end if;
  end process;

-- compare data to threshold
thresh_c: process(alt_count)
      begin
        if alt_count'event and alt_count = '1' then
          if thresh_comp(23 downto 16) > threshold(31 downto 24) then
            chrom11 <= '1';
          else
            chrom11 <= '0';
          end if;
          if thresh_comp(23 downto 16) < threshold(23 downto 16) then
            chrom1 <= '1';
          else
            chrom1 <= '0';
          end if;
          if thresh_comp(15 downto 8) > threshold(15 downto 8) then
            lum2 <= '1';
          else
            lum2 <= '0';
          end if;
          if thresh_comp(7 downto 0) > threshold(7 downto 0) then
            chrom2 <= '1';
          else
            chrom2 <= '0';
          end if;
        end if;
      end process;

thresh_comp2: process(alt_count)
      begin
        if alt_count'event and alt_count = '1' then
          if chrom11 = '1' and lum2 = '1' and chrom1 = '1' and chrom2 = '1'  then
            on_pix <= '1';
          else
            on_pix <= '0';
          end if;
        end if;
      end process;


-- wraparound line count to divide screen in four segments
line_manager: process(alt_count, line_count)
  begin
    if alt_count'event and alt_count = '1' then
      if line_count > 59 and line_count < 120 then
        line_count_seg <= line_count - 60;
        addr_marker <= "01";
      elsif line_count > 119 and line_count < 180 then
        line_count_seg <= line_count - 120;
        addr_marker <= "10";
      elsif line_count > 179 and line_count < 240 then
        line_count_seg  <= line_count - 180;
        addr_marker <= "11";
      else
        line_count_seg <= line_count;
        addr_marker <= "00";
      end if;
    end if;
  end process;

-- image split in 4X4 grid.  Count 'on' pixels in each box.
count_on1: process(alt_count, pix_count, line_count_seg, addr_marker)
  begin
    if alt_count'event and alt_count = '1' then
      if pix_count < 165 then
        grid1 <= grid1 + on_pix;
      elsif pix_count > 164 and pix_count < 325 then
        grid2 <= grid2 + on_pix;
      elsif pix_count > 324 and pix_count < 485 then
        grid3 <= grid3 + on_pix;
      elsif pix_count > 484  and pix_count < 645 then
        grid4 <= grid4 + on_pix;
      end if;
    
    if line_count_seg = 59 then
        if pix_count > 164 and pix_count < 168 then
           block_count <= grid1;
	   load <= "1" & addr_marker & "00";
	   grid1 <= X"0000";
         elsif pix_count > 324 and pix_count < 328 then
           block_count <= grid2;
           load <= "1" & addr_marker & "01";
	   grid2 <= X"0000";
         elsif pix_count > 484 and pix_count < 488 then
           block_count <= grid3;
           load <=  "1" & addr_marker & "10";
	   grid3 <= X"0000";
         elsif pix_count > 636 and pix_count < 640  then
           block_count <= grid4;
           load <=  "1" & addr_marker & "11";
	   grid4 <= X"0000";
         else
           block_count <= X"0000";
	   load <= "00000";
        end if;
     else
       load <= "00000";
      end if;
    end if;
  end process;

ICLK_OUT <= ICLK;
alt_count <= alt_counter(0);
n_count <= not alt_count;
data_in <= IPD & HPD;

end imp;
